# -*- coding: utf-8 -*-

from django.contrib import admin


class BaseOwnerInline(admin.StackedInline):
    raw_id_fields = ("organization_user",)


class BaseOrganizationAdmin(admin.ModelAdmin):
    list_display = ["name", "is_active"]
    prepopulated_fields = {"slug": ("name",)}
    search_fields = ["name"]
    list_filter = ("is_active",)


class BaseOrganizationUserAdmin(admin.ModelAdmin):
    list_display = ["user", "organization", "is_admin"]
    raw_id_fields = ("user", "organization")


class BaseOrganizationOwnerAdmin(admin.ModelAdmin):
    raw_id_fields = ("organization_user", "organization")



# -*- coding: utf-8 -*-

from django import forms
from django.contrib.auth import get_user_model
from django.contrib.sites.shortcuts import get_current_site
from django.utils.translation import gettext_lazy as _

from organizations.backends import invitation_backend
from organizations.models import Organization
from organizations.models import OrganizationUser
from organizations.utils import create_organization


class OrganizationForm(forms.ModelForm):
    """Form class for updating Organizations"""

    owner = forms.ModelChoiceField(OrganizationUser.objects.all())

    def __init__(self, request, *args, **kwargs):
        self.request = request
        super().__init__(*args, **kwargs)
        self.fields["owner"].queryset = self.instance.organization_users.filter(
            is_admin=True, user__is_active=True
        )
        self.fields["owner"].initial = self.instance.owner.organization_user

    class Meta:
        model = Organization
        exclude = ("users", "is_active")

    def save(self, commit=True):
        if self.instance.owner.organization_user != self.cleaned_data["owner"]:
            self.instance.change_owner(self.cleaned_data["owner"])
        return super().save(commit=commit)

    def clean_owner(self):
        owner = self.cleaned_data["owner"]
        if owner != self.instance.owner.organization_user:
            if self.request.user != self.instance.owner.organization_user.user:
                raise forms.ValidationError(
                    _("Only the organization owner can change ownerhip")
                )
        return owner


class OrganizationUserForm(forms.ModelForm):
    """Form class for updating OrganizationUsers"""

    class Meta:
        model = OrganizationUser
        exclude = ("organization", "user")

    def clean_is_admin(self):
        is_admin = self.cleaned_data["is_admin"]
        if (
            self.instance.organization.owner.organization_user == self.instance
            and not is_admin
        ):
            raise forms.ValidationError(_("The organization owner must be an admin"))
        return is_admin


class OrganizationUserAddForm(forms.ModelForm):
    """Form class for adding OrganizationUsers to an existing Organization"""

    email = forms.EmailField(max_length=75)

    def __init__(self, request, organization, *args, **kwargs):
        self.request = request
        self.organization = organization
        super().__init__(*args, **kwargs)

    class Meta:
        model = OrganizationUser
        exclude = ("user", "organization")

    def save(self, *args, **kwargs):
        """
        The save method should create a new OrganizationUser linking the User
        matching the provided email address. If not matching User is found it
        should kick off the registration process. It needs to create a User in
        order to link it to the Organization.
        """
        try:
            user = get_user_model().objects.get(
                email__iexact=self.cleaned_data["email"]
            )
        except get_user_model().DoesNotExist:
            user = invitation_backend().invite_by_email(
                self.cleaned_data["email"],
                **{
                    "domain": get_current_site(self.request),
                    "organization": self.organization,
                    "sender": self.request.user,
                }
            )
        # Send a notification email to this user to inform them that they
        # have been added to a new organization.
        invitation_backend().send_notification(
            user,
            **{
                "domain": get_current_site(self.request),
                "organization": self.organization,
                "sender": self.request.user,
            }
        )
        return OrganizationUser.objects.create(
            user=user,
            organization=self.organization,
            is_admin=self.cleaned_data["is_admin"],
        )

    def clean_email(self):
        email = self.cleaned_data["email"]
        if self.organization.users.filter(email__iexact=email).exists():
            raise forms.ValidationError(
                _("There is already an organization " "member with this email address!")
            )
        if get_user_model().objects.filter(email__iexact=email).count() > 1:
            raise forms.ValidationError(
                _("This email address has been used multiple times.")
            )
        return email


class OrganizationAddForm(forms.ModelForm):
    """
    Form class for creating a new organization, complete with new owner, including a
    User instance, OrganizationUser instance, and OrganizationOwner instance.
    """

    email = forms.EmailField(
        max_length=75, help_text=_("The email address for the account owner")
    )

    def __init__(self, request, *args, **kwargs):
        self.request = request
        super().__init__(*args, **kwargs)

    class Meta:
        model = Organization
        exclude = ("users", "is_active")

    def save(self, **kwargs):
        """
        Create the organization, then get the user, then make the owner.
        """
        is_active = True
        try:
            user = get_user_model().objects.get(email=self.cleaned_data["email"])
        except get_user_model().DoesNotExist:
            # TODO(bennylope): look into hooks for alt. registration systems here
            user = invitation_backend().invite_by_email(
                self.cleaned_data["email"],
                **{
                    "domain": get_current_site(self.request),
                    "organization": self.cleaned_data["name"],
                    "sender": self.request.user,
                    "created": True,
                }
            )
            is_active = False
        return create_organization(
            user,
            self.cleaned_data["name"],
            self.cleaned_data["slug"],
            is_active=is_active,
        )


class SignUpForm(forms.Form):
    """
    Form class for signing up a new user and new account.
    """

    name = forms.CharField(max_length=50, help_text=_("The name of the organization"))
    slug = forms.SlugField(
        max_length=50,
        help_text=_("The name in all lowercase, suitable for URL identification"),
    )
    email = forms.EmailField()



# -*- coding: utf-8 -*-

from organizations.abstract import AbstractOrganization
from organizations.abstract import AbstractOrganizationInvitation
from organizations.abstract import AbstractOrganizationOwner
from organizations.abstract import AbstractOrganizationUser


class Organization(AbstractOrganization):
    """
    Default Organization model.
    """

    class Meta(AbstractOrganization.Meta):
        abstract = False


class OrganizationUser(AbstractOrganizationUser):
    """
    Default OrganizationUser model.
    """

    class Meta(AbstractOrganizationUser.Meta):
        abstract = False


class OrganizationOwner(AbstractOrganizationOwner):
    """
    Default OrganizationOwner model.
    """

    class Meta(AbstractOrganizationOwner.Meta):
        abstract = False


class OrganizationInvitation(AbstractOrganizationInvitation):
    class Meta(AbstractOrganizationInvitation.Meta):
        abstract = False



# -*- coding: utf-8 -*-

from django.contrib import admin

from organizations import models
from organizations.base_admin import BaseOrganizationAdmin
from organizations.base_admin import BaseOrganizationOwnerAdmin
from organizations.base_admin import BaseOrganizationUserAdmin
from organizations.base_admin import BaseOwnerInline


class OwnerInline(BaseOwnerInline):
    model = models.OrganizationOwner


@admin.register(models.Organization)
class OrganizationAdmin(BaseOrganizationAdmin):
    inlines = [OwnerInline]


@admin.register(models.OrganizationUser)
class OrganizationUserAdmin(BaseOrganizationUserAdmin):
    pass


@admin.register(models.OrganizationOwner)
class OrganizationOwnerAdmin(BaseOrganizationOwnerAdmin):
    pass


@admin.register(models.OrganizationInvitation)
class OrganizationInvitationAdmin(admin.ModelAdmin):
    pass



# Generated by Django 4.0.7 on 2022-09-30 17:00

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0012_alter_user_first_name_max_length'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('organizations', '0004_organizationinvitation'),
    ]

    operations = [
        migrations.AlterField(
            model_name='organization',
            name='users',
            field=models.ManyToManyField(related_name='%(app_label)s_%(class)s', through='organizations.OrganizationUser', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='organizationinvitation',
            name='invited_by',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='%(app_label)s_%(class)s_sent_invitations', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='organizationinvitation',
            name='invitee',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='%(app_label)s_%(class)s_invitations', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='organizationuser',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='%(app_label)s_%(class)s', to=settings.AUTH_USER_MODEL),
        ),
    ]



# Generated by Django 2.0.5 on 2019-06-27 00:54

import django.db.models.deletion
import django.utils.timezone
from django.conf import settings
from django.db import migrations
from django.db import models

import organizations.base
import organizations.fields


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("organizations", "0003_field_fix_and_editable"),
    ]

    operations = [
        migrations.CreateModel(
            name="OrganizationInvitation",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("guid", models.UUIDField(editable=False)),
                (
                    "invitee_identifier",
                    models.CharField(
                        help_text="The contact identifier for the invitee, email, phone number, social media handle, etc.",
                        max_length=1000,
                    ),
                ),
                (
                    "created",
                    organizations.fields.AutoCreatedField(
                        default=django.utils.timezone.now, editable=False
                    ),
                ),
                (
                    "modified",
                    organizations.fields.AutoLastModifiedField(
                        default=django.utils.timezone.now, editable=False
                    ),
                ),
                (
                    "invited_by",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="organizations_organizationinvitation_sent_invitations",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "invitee",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="organizations_organizationinvitation_invitations",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "organization",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="organization_invites",
                        to="organizations.Organization",
                    ),
                ),
            ],
            options={"abstract": False},
            bases=(organizations.base.UnicodeMixin, models.Model),
        )
    ]



# -*- coding: utf-8 -*-

from django import forms
from django.contrib.auth.forms import UserCreationForm


class UserRegistrationForm(UserCreationForm):
    """
    Form class for completing a user's registration and activating the
    User.

    The class operates on a user model which is assumed to have the required
    fields of a BaseUserModel
    """

    # TODO(bennylope): Remove this entirely and replace with base class


def org_registration_form(org_model):
    """
    Generates a registration ModelForm for the given organization model class
    """

    class OrganizationRegistrationForm(forms.ModelForm):
        """Form class for creating new organizations owned by new users."""

        email = forms.EmailField()

        class Meta:
            model = org_model
            exclude = ("is_active", "users")

        # def save(self, *args, **kwargs):
        #     self.instance.is_active = False
        #     super().save(*args, **kwargs)

    return OrganizationRegistrationForm



# -*- coding: utf-8 -*-

"""Backend classes should provide common interface"""

import email.utils
import inspect
import uuid
from typing import ClassVar  # noqa
from typing import Optional  # noqa
from typing import Text  # noqa

from django.conf import settings
from django.contrib.auth import authenticate
from django.contrib.auth import get_user_model
from django.contrib.auth import login
from django.contrib.auth.tokens import PasswordResetTokenGenerator
from django.core.mail import EmailMessage
from django.http import Http404
from django.shortcuts import redirect
from django.shortcuts import render
from django.template import loader
from django.urls import path
from django.urls import reverse
from django.utils.translation import gettext as _

from organizations.backends.forms import UserRegistrationForm
from organizations.backends.forms import org_registration_form
from organizations.utils import create_organization
from organizations.utils import default_org_model
from organizations.utils import model_field_attr


class BaseBackend:
    """
    Base backend class for registering and inviting users to an organization
    """

    registration_form_template = "organizations/register_form.html"
    activation_success_template = "organizations/register_success.html"

    def __init__(self, org_model=None, namespace=None):
        # type: (Optional[ClassVar], Optional[Text]) -> None
        self.user_model = get_user_model()
        self.org_model = org_model or default_org_model()
        self.namespace = namespace

    def namespace_preface(self):
        return "" if not self.namespace else "{}:".format(self.namespace)

    def get_urls(self):
        raise NotImplementedError

    def get_success_url(self):
        """Will return the class's `success_url` attribute unless overridden"""
        raise NotImplementedError

    def get_form(self, **kwargs):
        """Returns the form for registering or inviting a user"""
        if not getattr(self, "form_class", None):
            raise AttributeError("You must define a form_class")
        return self.form_class(**kwargs)

    def get_token(self, user, **kwargs):
        """Returns a unique token for the given user"""
        return PasswordResetTokenGenerator().make_token(user)

    def get_username(self):
        """
        Returns a UUID-based 'random' and unique username.

        This is required data for user models with a username field.
        """
        return str(uuid.uuid4())[
            : model_field_attr(self.user_model, "username", "max_length")
        ]

    def activate_organizations(self, user):
        """
        Activates the related organizations for the user.

        It only activates the related organizations by model type - that is, if
        there are multiple types of organizations then only organizations in
        the provided model class are activated.
        """
        try:
            relation_name = self.org_model().user_relation_name
        except TypeError:
            # No org_model specified, raises a TypeError because NoneType is
            # not callable. This the most sensible default:
            relation_name = "organizations_organization"
        organization_set = getattr(user, relation_name)
        for org in organization_set.filter(is_active=False):
            org.is_active = True
            org.save()

    def activate_view(self, request, user_id, token):
        """
        View function that activates the given User by setting `is_active` to
        true if the provided information is verified.
        """
        try:
            user = self.user_model.objects.get(id=user_id, is_active=False)
        except self.user_model.DoesNotExist:
            raise Http404(_("Your URL may have expired."))

        if not PasswordResetTokenGenerator().check_token(user, token):
            raise Http404(_("Your URL may have expired."))
        form = self.get_form(
            data=request.POST or None, files=request.FILES or None, instance=user
        )
        if form.is_valid():
            form.instance.is_active = True
            user = form.save()
            user.set_password(form.cleaned_data["password1"])
            user.save()
            self.activate_organizations(user)
            user = authenticate(
                username=form.cleaned_data["username"],
                password=form.cleaned_data["password1"],
            )
            if user is None:
                raise Http404(_("Can't authenticate user"))
            login(request, user)
            return redirect(self.get_success_url())
        return render(request, self.registration_form_template, {"form": form})

    def send_reminder(self, user, sender=None, **kwargs):
        """Sends a reminder email to the specified user"""
        if user.is_active:
            return False
        token = PasswordResetTokenGenerator().make_token(user)
        kwargs.update({"token": token})
        self.email_message(
            user, self.reminder_subject, self.reminder_body, sender, **kwargs
        ).send()

    def email_message(
        self,
        user,
        subject_template,
        body_template,
        sender=None,
        message_class=EmailMessage,
        **kwargs
    ):
        """
        Returns an email message for a new user. This can be easily overridden.
        For instance, to send an HTML message, use the EmailMultiAlternatives message_class
        and attach the additional conent.
        """
        if sender:
            try:
                display_name = sender.get_full_name()
            except (AttributeError, TypeError):
                display_name = sender.get_username()
            from_email = "%s <%s>" % (
                display_name,
                email.utils.parseaddr(settings.DEFAULT_FROM_EMAIL)[1],
            )
            reply_to = "%s <%s>" % (display_name, sender.email)
        else:
            from_email = settings.DEFAULT_FROM_EMAIL
            reply_to = from_email

        headers = {"Reply-To": reply_to}
        kwargs.update({"sender": sender, "user": user})

        subject_template = loader.get_template(subject_template)
        body_template = loader.get_template(body_template)
        subject = subject_template.render(
            kwargs
        ).strip()  # Remove stray newline characters
        body = body_template.render(kwargs)
        return message_class(subject, body, from_email, [user.email], headers=headers)


class RegistrationBackend(BaseBackend):
    """
    A backend for allowing new users to join the site by creating a new user
    associated with a new organization.
    """

    # NOTE this backend stands to be simplified further, as email verification
    # should be beyond the purview of this app
    activation_subject = "organizations/email/activation_subject.txt"
    activation_body = "organizations/email/activation_body.html"
    reminder_subject = "organizations/email/reminder_subject.txt"
    reminder_body = "organizations/email/reminder_body.html"
    form_class = UserRegistrationForm

    def get_success_url(self):
        return reverse("registration_success")

    def get_urls(self):
        return [
            path("complete/", view=self.success_view, name="registration_success"),
            path(
                "<int:user_id>-<token>/",
                view=self.activate_view,
                name="registration_register",
            ),
            path("", view=self.create_view, name="registration_create"),
        ]

    @property
    def urls(self):
        return self.get_urls(), self.namespace or "registration"

    def register_by_email(self, email, sender=None, request=None, **kwargs):
        """
        Returns a User object filled with dummy data and not active, and sends
        an invitation email.
        """
        try:
            user = self.user_model.objects.get(email=email)
        except self.user_model.DoesNotExist:
            user = self.user_model.objects.create(
                username=self.get_username(),
                email=email,
                password=self.user_model.objects.make_random_password(),
            )
            user.is_active = False
            user.save()
        self.send_activation(user, sender, **kwargs)
        return user

    def send_activation(self, user, sender=None, **kwargs):
        """
        Invites a user to join the site
        """
        if user.is_active:
            return False
        token = self.get_token(user)
        kwargs.update({"token": token})
        self.email_message(
            user, self.activation_subject, self.activation_body, sender, **kwargs
        ).send()

    def create_view(self, request):
        """
        Initiates the organization and user account creation process
        """
        if request.user.is_authenticated:
            return redirect("organization_add")
        form = org_registration_form(self.org_model)(request.POST or None)
        if form.is_valid():
            try:
                user = self.user_model.objects.get(email=form.cleaned_data["email"])
            except self.user_model.DoesNotExist:
                user = self.user_model.objects.create(
                    username=self.get_username(),
                    email=form.cleaned_data["email"],
                    password=self.user_model.objects.make_random_password(),
                )
                user.is_active = False
                user.save()
            else:
                return redirect("organization_add")
            organization = create_organization(
                user,
                form.cleaned_data["name"],
                form.cleaned_data["slug"],
                is_active=False,
            )
            return render(
                request,
                self.activation_success_template,
                {"user": user, "organization": organization},
            )
        return render(request, self.registration_form_template, {"form": form})

    def success_view(self, request):
        return render(request, self.activation_success_template, {})


class InvitationBackend(BaseBackend):
    """
    A backend for inviting new users to join the site as members of an
    organization.
    """

    notification_subject = "organizations/email/notification_subject.txt"
    notification_body = "organizations/email/notification_body.html"
    invitation_subject = "organizations/email/invitation_subject.txt"
    invitation_body = "organizations/email/invitation_body.html"
    reminder_subject = "organizations/email/reminder_subject.txt"
    reminder_body = "organizations/email/reminder_body.html"
    form_class = UserRegistrationForm

    def get_success_url(self):
        # TODO get this url name from an attribute
        return reverse("organization_list")

    def get_urls(self):
        # TODO enable naming based on a model?
        return [
            path(
                "<int:user_id>-<token>/",
                view=self.activate_view,
                name="invitations_register",
            )
        ]

    def invite_by_email(self, email, sender=None, request=None, **kwargs):
        """Creates an inactive user with the information we know and then sends
        an invitation email for that user to complete registration.

        If your project uses email in a different way then you should make to
        extend this method as it only checks the `email` attribute for Users.
        """
        try:
            user = self.user_model.objects.get(email=email)
        except self.user_model.DoesNotExist:
            # TODO break out user creation process
            if (
                "username"
                in inspect.getfullargspec(self.user_model.objects.create_user).args
            ):
                user = self.user_model.objects.create(
                    username=self.get_username(),
                    email=email,
                    password=self.user_model.objects.make_random_password(),
                )
            else:
                user = self.user_model.objects.create(
                    email=email, password=self.user_model.objects.make_random_password()
                )
            user.is_active = False
            user.save()
        self.send_invitation(user, sender, **kwargs)
        return user

    def send_invitation(self, user, sender=None, **kwargs):
        """An intermediary function for sending an invitation email that
        selects the templates, generating the token, and ensuring that the user
        has not already joined the site.
        """
        if user.is_active:
            return False
        token = self.get_token(user)
        kwargs.update({"token": token})
        self.email_message(
            user, self.invitation_subject, self.invitation_body, sender, **kwargs
        ).send()
        return True

    def send_notification(self, user, sender=None, **kwargs):
        """
        An intermediary function for sending an notification email informing
        a pre-existing, active user that they have been added to a new
        organization.
        """
        if not user.is_active:
            return False
        self.email_message(
            user, self.notification_subject, self.notification_body, sender, **kwargs
        ).send()
        return True



# -*- coding: utf-8 -*-

from itertools import chain


def default_org_model():
    """Encapsulates importing the concrete model"""
    from organizations.models import Organization

    return Organization


def model_field_names(model):
    """
    Returns a list of field names in the model

    Direct from Django upgrade migration guide.
    """
    return list(
        set(
            chain.from_iterable(
                (field.name, field.attname)
                if hasattr(field, "attname")
                else (field.name,)
                for field in model._meta.get_fields()
                if not (field.many_to_one and field.related_model is None)
            )
        )
    )


def create_organization(
    user,
    name,
    slug=None,
    is_active=None,
    org_defaults=None,
    org_user_defaults=None,
    **kwargs
):
    """
    Returns a new organization, also creating an initial organization user who
    is the owner.

    The specific models can be specified if a custom organization app is used.
    The simplest way would be to use a partial.

    >>> from organizations.utils import create_organization
    >>> from myapp.models import Account
    >>> from functools import partial
    >>> create_account = partial(create_organization, model=Account)

    """
    org_model = (
        kwargs.pop("model", None)
        or kwargs.pop("org_model", None)
        or default_org_model()
    )
    kwargs.pop("org_user_model", None)  # Discard deprecated argument

    org_owner_model = org_model.owner.related.related_model
    org_user_model = org_model.organization_users.rel.related_model

    if org_defaults is None:
        org_defaults = {}
    if org_user_defaults is None:
        if "is_admin" in model_field_names(org_user_model):
            org_user_defaults = {"is_admin": True}
        else:
            org_user_defaults = {}

    if slug is not None:
        org_defaults.update({"slug": slug})
    if is_active is not None:
        org_defaults.update({"is_active": is_active})

    org_defaults.update({"name": name})
    organization = org_model.objects.create(**org_defaults)

    org_user_defaults.update({"organization": organization, "user": user})
    new_user = org_user_model.objects.create(**org_user_defaults)

    org_owner_model.objects.create(
        organization=organization, organization_user=new_user
    )
    return organization


def model_field_attr(model, model_field, attr):
    """
    Returns the specified attribute for the specified field on the model class.
    """
    fields = dict([(field.name, field) for field in model._meta.fields])
    return getattr(fields[model_field], attr)



# Generated by Django 4.0.7 on 2022-10-13 02:45

from django.db import migrations
import organizations.fields


class Migration(migrations.Migration):

    dependencies = [
        ('organizations', '0005_alter_organization_users_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='organization',
            name='slug',
            field=organizations.fields.SlugField(blank=True, editable=False, help_text='The name in all lowercase, suitable for URL identification', max_length=200, populate_from='name', unique=True),
        ),
    ]



# -*- coding: utf-8 -*-

from django.contrib.sites.shortcuts import get_current_site
from django.core.exceptions import ImproperlyConfigured
from django.http import HttpResponseGone
from django.shortcuts import redirect
from django.urls import reverse
from django.utils.translation import gettext as _
from django.views.generic import CreateView
from django.views.generic import DeleteView
from django.views.generic import DetailView
from django.views.generic import FormView
from django.views.generic import ListView
from django.views.generic import UpdateView

from organizations.backends import invitation_backend
from organizations.backends import registration_backend
from organizations.forms import OrganizationAddForm
from organizations.forms import OrganizationForm
from organizations.forms import OrganizationUserAddForm
from organizations.forms import OrganizationUserForm
from organizations.forms import SignUpForm
from organizations.utils import create_organization
from organizations.views.mixins import OrganizationMixin
from organizations.views.mixins import OrganizationUserMixin


class BaseOrganizationList(ListView):
    context_object_name = "organizations"

    def get_queryset(self):
        return self.org_model.active.filter(users=self.request.user)


class BaseOrganizationDetail(OrganizationMixin, DetailView):
    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["organization_users"] = self.organization.organization_users.all()
        context["organization"] = self.organization
        return context


class BaseOrganizationCreate(CreateView):
    form_class = OrganizationAddForm
    template_name = "organizations/organization_form.html"

    def get_success_url(self):
        return reverse("organization_list")

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs.update({"request": self.request})
        return kwargs


class BaseOrganizationUpdate(OrganizationMixin, UpdateView):
    form_class = OrganizationForm

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs.update({"request": self.request})
        return kwargs


class BaseOrganizationDelete(OrganizationMixin, DeleteView):
    def get_success_url(self):
        return reverse("organization_list")


class BaseOrganizationUserList(OrganizationMixin, ListView):
    def get(self, request, *args, **kwargs):
        self.organization = self.get_organization()
        self.object_list = self.organization.organization_users.all()
        context = self.get_context_data(
            object_list=self.object_list,
            organization_users=self.object_list,
            organization=self.organization,
        )
        return self.render_to_response(context)


class BaseOrganizationUserDetail(OrganizationUserMixin, DetailView):
    pass


class BaseOrganizationUserCreate(OrganizationMixin, CreateView):
    form_class = OrganizationUserAddForm
    template_name = "organizations/organizationuser_form.html"

    def get_success_url(self):
        return reverse(
            "organization_user_list",
            kwargs={"organization_pk": self.object.organization.pk},
        )

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs.update({"organization": self.organization, "request": self.request})
        return kwargs

    def get(self, request, *args, **kwargs):
        self.organization = self.get_object()
        return super().get(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        self.organization = self.get_object()
        return super().post(request, *args, **kwargs)


class BaseOrganizationUserRemind(OrganizationUserMixin, DetailView):
    """
    Reminder view for already-linked org users

    This is only applicable for invitation backends using the
    strategy the original "default" backend uses, which is to
    immediately add existing users to the organization after
    invite, but leave new users inactive until confirmation.

    """

    template_name = "organizations/organizationuser_remind.html"
    # TODO move to invitations backend?

    def get_success_url(self):
        return reverse(
            "organization_user_list",
            kwargs={"organization_pk": self.object.organization.pk},
        )

    def get(self, request, *args, **kwargs):
        self.object = self.get_object()
        if self.object.user.is_active:
            return HttpResponseGone(_("User is already active"))
        return super().get(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        self.object = self.get_object()
        if self.object.user.is_active:
            return HttpResponseGone(_("User is already active"))
        invitation_backend().send_reminder(
            self.object.user,
            **{
                "domain": get_current_site(self.request),
                "organization": self.organization,
                "sender": request.user,
            }
        )
        return redirect(self.get_success_url())


class BaseOrganizationUserUpdate(OrganizationUserMixin, UpdateView):
    form_class = OrganizationUserForm


class BaseOrganizationUserDelete(OrganizationUserMixin, DeleteView):
    def get_success_url(self):
        return reverse(
            "organization_user_list",
            kwargs={"organization_pk": self.object.organization.pk},
        )


class OrganizationSignup(FormView):
    """
    View that allows unregistered users to create an organization account.

    It simply processes the form and then calls the specified registration
    backend.
    """

    form_class = SignUpForm
    template_name = "organizations/signup_form.html"
    # TODO get success from backend, because some backends may do something
    # else, like require verification
    backend = registration_backend()

    def dispatch(self, request, *args, **kwargs):
        if request.user.is_authenticated:
            return redirect("organization_add")
        return super().dispatch(request, *args, **kwargs)

    def get_success_url(self):
        if getattr(self, "success_url", None):
            return self.success_url
        raise ImproperlyConfigured(
            "{cls} must either have a `success_url` attribute defined"
            "or override `get_success_url`".format(cls=self.__class__.__name__)
        )

    def form_valid(self, form):
        """
        Register user and create the organization
        """
        user = self.backend.register_by_email(form.cleaned_data["email"])
        create_organization(
            user=user,
            name=form.cleaned_data["name"],
            slug=form.cleaned_data["slug"],
            is_active=False,
        )
        return redirect(self.get_success_url())


class ViewFactory:
    """
    A class that can create a faked 'module' with model specific views

    These views have NO access control applied.
    """

    def __init__(self, org_model):
        self.org_model = org_model

    @property
    def OrganizationList(self):
        klass = BaseOrganizationList
        klass.org_model = self.org_model
        return klass

    @property
    def OrganizationDetail(self):
        klass = BaseOrganizationDetail
        klass.org_model = self.org_model
        return klass

    @property
    def OrganizationCreate(self):
        klass = BaseOrganizationCreate
        klass.org_model = self.org_model
        return klass

    @property
    def OrganizationUpdate(self):
        klass = BaseOrganizationUpdate
        klass.org_model = self.org_model
        return klass

    @property
    def OrganizationDelete(self):
        klass = BaseOrganizationDelete
        klass.org_model = self.org_model
        return klass

    @property
    def OrganizationUserList(self):
        klass = BaseOrganizationUserList
        klass.org_model = self.org_model
        return klass

    @property
    def OrganizationUserDetail(self):
        klass = BaseOrganizationUserDetail
        klass.org_model = self.org_model
        return klass

    @property
    def OrganizationUserUpdate(self):
        klass = BaseOrganizationUserUpdate
        klass.org_model = self.org_model
        return klass

    @property
    def OrganizationUserCreate(self):
        klass = BaseOrganizationUserCreate
        klass.org_model = self.org_model
        return klass

    @property
    def OrganizationUserDelete(self):
        klass = BaseOrganizationUserDelete
        klass.org_model = self.org_model
        return klass

    @property
    def OrganizationUserRemind(self):
        klass = BaseOrganizationUserRemind
        klass.org_model = self.org_model
        return klass



# -*- coding: utf-8 -*-

"""
Invitations that use an invitation model
"""
import email.utils
from typing import List  # noqa
from typing import Optional  # noqa
from typing import Text  # noqa
from typing import Tuple  # noqa

from django.conf import settings
from django.contrib.auth.models import AbstractUser  # noqa
from django.core.mail import EmailMessage
from django.http import HttpRequest  # noqa
from django.http import HttpResponse  # noqa
from django.http import HttpResponseForbidden
from django.shortcuts import get_object_or_404
from django.shortcuts import redirect
from django.shortcuts import render
from django.template import loader
from django.urls import path
from django.utils.translation import gettext_lazy as _

from organizations.backends.defaults import InvitationBackend
from organizations.backends.forms import UserRegistrationForm
from organizations.base import AbstractBaseOrganization  # noqa
from organizations.base import OrganizationInvitationBase  # noqa


class ModelInvitation(InvitationBackend):
    """Invitation backend for model-tracked invitations"""

    notification_subject = "organizations/email/notification_subject.txt"
    notification_body = "organizations/email/notification_body.html"
    invitation_subject = "organizations/email/modeled_invitation_subject.txt"
    invitation_body = "organizations/email/modeled_invitation_body.html"
    reminder_subject = "organizations/email/modeled_reminder_subject.txt"
    reminder_body = "organizations/email/modeled_reminder_body.html"
    invitation_join_template = "organizations/invitation_join.html"

    form_class = UserRegistrationForm

    def __init__(self, org_model=None, namespace=None):
        super().__init__(org_model=org_model, namespace=namespace)
        self.invitation_model = (
            self.org_model.invitation_model
        )  # type: OrganizationInvitationBase

    def get_invitation_queryset(self):
        """Return this to use a custom queryset that checks for expiration, for example"""
        return self.invitation_model.objects.all()

    def get_invitation_accepted_url(self):
        """Returns the redirect URL after user accepts invitation"""
        return "/"

    def get_invitation_accepted_registered_url(self):
        """Returns the redirect URL after new user accepts invitation"""
        return self.get_invitation_accepted_url()

    def activation_router(self, request, guid):
        """"""
        invitation = get_object_or_404(self.get_invitation_queryset(), guid=guid)
        if invitation.invitee:
            return redirect(self.get_invitation_accepted_url())

        if request.user.is_authenticated:
            return self.activate_existing_user_view(request, invitation)
        else:
            return self.activate_new_user_view(request, invitation)

    def activate_existing_user_view(self, request, invitation):
        # type: (HttpRequest, OrganizationInvitationBase) -> HttpResponse
        """"""
        if request.user == invitation.invited_by:
            return HttpResponseForbidden(_("This is not your invitation"))
        if request.method == "POST":
            invitation.activate(request.user)
            return redirect(self.get_invitation_accepted_url())
        return render(
            request, self.invitation_join_template, {"invitation": invitation}
        )

    def activate_new_user_view(self, request, invitation):
        # type: (HttpRequest, OrganizationInvitationBase) -> HttpResponse
        """"""
        form = self.get_form(data=request.POST or None)
        if request.method == "POST" and form.is_valid():
            new_user = form.save()  # type: AbstractUser
            invitation.activate(new_user)
            return redirect(self.get_invitation_accepted_registered_url())
        return render(
            request,
            self.registration_form_template,
            {"invitation": invitation, "form": form},
        )

    def get_urls(self):
        # type: () -> List[path]
        return [
            path(
                "<uuid:guid>/", view=self.activation_router, name="invitations_register"
            )
        ]

    @property
    def urls(self):
        # type: () -> Tuple[List[path], Text]
        return self.get_urls(), self.namespace or "registration"

    def invite_by_email(self, email, user, organization, **kwargs):
        """
        Primary interface method by which one user invites another to join

        Args:
            email:
            request:
            **kwargs:

        Returns:
            an invitation instance

        Raises:
            MultipleObjectsReturned if multiple matching users are found

        """
        # TODO(bennylope): verify no such user already?
        # try:
        #     invitee = self.user_model.objects.get(email__iexact=email)
        # except self.user_model.DoesNotExist:
        #     invitee = None

        # TODO allow sending just the OrganizationUser instance
        user_invitation = self.invitation_model.objects.create(
            invitee_identifier=email.lower(),
            invited_by=user,
            organization=organization,
        )
        self.send_invitation(user_invitation)
        return user_invitation

    def send_invitation(self, invitation, **kwargs):
        """
        Sends an invitation message for a specific invitation.

        This could be overridden to do other things, such as sending a confirmation
        email to the sender.

        Args:
            invitation:

        Returns:

        """
        return self.email_message(
            invitation.invitee_identifier,
            self.invitation_subject,
            self.invitation_body,
            invitation.invited_by,
            **kwargs
        ).send()

    def email_message(
        self,
        recipient,  # type: Text
        subject_template,  # type: Text
        body_template,  # type: Text
        sender=None,  # type: Optional[AbstractUser]
        message_class=EmailMessage,
        **kwargs
    ):
        """
        Returns an invitation email message. This can be easily overridden.
        For instance, to send an HTML message, use the EmailMultiAlternatives message_class
        and attach the additional conent.
        """
        from_email = "%s %s <%s>" % (
            sender.first_name,
            sender.last_name,
            email.utils.parseaddr(settings.DEFAULT_FROM_EMAIL)[1],
        )
        reply_to = "%s %s <%s>" % (sender.first_name, sender.last_name, sender.email)

        headers = {"Reply-To": reply_to}
        kwargs.update({"sender": sender, "recipient": recipient})

        subject_template = loader.get_template(subject_template)
        body_template = loader.get_template(body_template)

        subject = subject_template.render(
            kwargs
        ).strip()  # Remove stray newline characters

        body = body_template.render(kwargs)

        return message_class(subject, body, from_email, [recipient], headers=headers)



# -*- coding: utf-8 -*-

from django.conf import settings
from django.contrib.auth.models import User

from organizations.utils import model_field_attr

ORGS_INVITATION_BACKEND = getattr(
    settings, "INVITATION_BACKEND", "organizations.backends.defaults.InvitationBackend"
)

ORGS_REGISTRATION_BACKEND = getattr(
    settings,
    "REGISTRATION_BACKEND",
    "organizations.backends.defaults.RegistrationBackend",
)

ORGS_EMAIL_LENGTH = model_field_attr(User, "email", "max_length")



# -*- coding: utf-8 -*-

from django.core.exceptions import PermissionDenied
from django.shortcuts import get_object_or_404
from django.utils.functional import cached_property
from django.utils.translation import gettext_lazy as _

from organizations.models import Organization
from organizations.models import OrganizationUser


class OrganizationMixin:
    """Mixin used like a SingleObjectMixin to fetch an organization"""

    org_model = Organization
    org_context_name = "organization"

    def get_org_model(self):
        return self.org_model

    def get_context_data(self, **kwargs):
        kwargs.update({self.org_context_name: self.organization})
        return super().get_context_data(**kwargs)

    @cached_property
    def organization(self):
        organization_pk = self.kwargs.get("organization_pk", None)
        return get_object_or_404(self.get_org_model(), pk=organization_pk)

    def get_object(self):
        return self.organization

    get_organization = get_object  # Now available when `get_object` is overridden


class OrganizationUserMixin(OrganizationMixin):
    """Mixin used like a SingleObjectMixin to fetch an organization user"""

    user_model = OrganizationUser
    org_user_context_name = "organization_user"

    def get_user_model(self):
        return self.user_model

    def get_context_data(self, **kwargs):
        kwargs = super().get_context_data(**kwargs)
        kwargs.update(
            {
                self.org_user_context_name: self.object,
                self.org_context_name: self.object.organization,
            }
        )
        return kwargs

    @cached_property
    def organization_user(self):
        """
        Returns the OrganizationUser object

        This is fetched based on the primary keys for both
        the organization and the organization user.
        """
        organization_pk = self.kwargs.get("organization_pk", None)
        user_pk = self.kwargs.get("user_pk", None)
        return get_object_or_404(
            self.get_user_model().objects.select_related(),
            user__pk=user_pk,
            organization__pk=organization_pk,
        )

    def get_object(self):
        """Proxy for the base class interface

        This can be called all day long and the object is queried once.
        """
        return self.organization_user


class MembershipRequiredMixin:
    """This mixin presumes that authentication has already been checked"""

    def dispatch(self, request, *args, **kwargs):
        self.request = request
        self.args = args
        self.kwargs = kwargs
        if (
            not self.organization.is_member(request.user)
            and not request.user.is_superuser
        ):
            raise PermissionDenied(_("Wrong organization"))
        return super().dispatch(request, *args, **kwargs)


class AdminRequiredMixin:
    """This mixin presumes that authentication has already been checked"""

    def dispatch(self, request, *args, **kwargs):
        self.request = request
        self.args = args
        self.kwargs = kwargs
        if (
            not self.organization.is_admin(request.user)
            and not request.user.is_superuser
        ):
            raise PermissionDenied(_("Sorry, admins only"))
        return super().dispatch(request, *args, **kwargs)


class OwnerRequiredMixin:
    """This mixin presumes that authentication has already been checked"""

    def dispatch(self, request, *args, **kwargs):
        self.request = request
        self.args = args
        self.kwargs = kwargs
        if (
            self.organization.owner.organization_user.user != request.user
            and not request.user.is_superuser
        ):
            raise PermissionDenied(_("You are not the organization owner"))
        return super().dispatch(request, *args, **kwargs)
